//
//  CPDFAnnotationSampleView.m
//  compdfkit-tools
//
//  Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
//
//  THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
//  AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
//  UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
//  This notice may not be removed from this file.
//

#import "CPDFAnnotationSampleView.h"

#define PI 3.14159265358979323846

@interface CPDFAnnotationSampleView ()

@property (nonatomic, assign) CGRect centerRect;

@property (nonatomic, assign) CGRect arrowRect;

@end

@implementation CPDFAnnotationSampleView

#pragma mark - Initializers

- (instancetype)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        
    }
    return self;
}

- (void)drawRect:(CGRect)rect {
    [super drawRect:rect];
    CGContextRef context = UIGraphicsGetCurrentContext();
    self.centerRect = CGRectInset(rect, 130, 40);
    self.arrowRect = CGRectInset(rect, 100, 20);
    
    [self drawSamples:context];
}

#pragma mark - Private Methods

- (void)drawSamples:(CGContextRef)context {
    CGContextSetAlpha(context, self.opcity);
    switch (self.selecIndex) {
        case CPDFSamplesNote:
        {
            CGContextSetFillColorWithColor(context, self.fillColor.CGColor);
            CGContextFillRect(context, self.centerRect);
            UIImage *image = [UIImage imageNamed:@"CPDFAnnotationBarImageNote" inBundle:[NSBundle bundleForClass:self.class] compatibleWithTraitCollection:nil];
            [image drawInRect:self.centerRect];
        }
            break;
        case CPDFSamplesHighlight:
        {
            CGContextSetFillColorWithColor(context, self.fillColor.CGColor);
            CGContextFillRect(context, self.centerRect);
            UIImage *image = [UIImage imageNamed:@"CPDFAnnotationBarImageHighLight" inBundle:[NSBundle bundleForClass:self.class] compatibleWithTraitCollection:nil];
            [image drawInRect:self.centerRect];
        }
            break;
        case CPDFSamplesUnderline:
        {
            CGContextSetFillColorWithColor(context, self.fillColor.CGColor);
            CGRect strikeoutRect = CGRectInset(self.centerRect, 0, 18);
            CGRect underLineRect = CGRectOffset(strikeoutRect, 0, 18);
            CGContextFillRect(context, underLineRect);
            UIImage *image = [UIImage imageNamed:@"CPDFAnnotationBarImageHighLight" inBundle:[NSBundle bundleForClass:self.class] compatibleWithTraitCollection:nil];
            [image drawInRect:self.centerRect];
        }
            break;
        case CPDFSamplesStrikeout:
        {
            CGContextSetFillColorWithColor(context, self.fillColor.CGColor);
            CGRect strikeoutRect = CGRectInset(self.centerRect, 0, 18);
            CGContextFillRect(context, strikeoutRect);
            UIImage *image = [UIImage imageNamed:@"CPDFAnnotationBarImageHighLight" inBundle:[NSBundle bundleForClass:self.class] compatibleWithTraitCollection:nil];
            [image drawInRect:self.centerRect];
        }
        case CPDFSamplesSquiggly:
        {
            CGContextSetStrokeColorWithColor(context, self.fillColor.CGColor);
            
            float tWidth = self.centerRect.size.width / 6.0;
            CGContextMoveToPoint(context, CGRectGetMinX(self.centerRect), CGRectGetMaxY(self.centerRect));
            CGContextSetLineWidth(context, 2.0);
            CGContextAddCurveToPoint(context,
                                     CGRectGetMinX(self.centerRect)+tWidth,CGRectGetMaxY(self.centerRect)+4,
                                     CGRectGetMinX(self.centerRect)+tWidth*2.0,CGRectGetMaxY(self.centerRect)-4,
                                     CGRectGetMinX(self.centerRect)+tWidth*3.0,CGRectGetMaxY(self.centerRect));
            CGContextAddCurveToPoint(context,
                                     CGRectGetMinX(self.centerRect)+tWidth*4.0,CGRectGetMaxY(self.centerRect)+4,
                                     CGRectGetMinX(self.centerRect)+tWidth*5.0,CGRectGetMaxY(self.centerRect)-4,
                                     CGRectGetMinX(self.centerRect)+tWidth*6.0,CGRectGetMaxY(self.centerRect));
            
            CGContextStrokePath(context);
            
            UIImage *image = [UIImage imageNamed:@"CPDFAnnotationBarImageHighLight" inBundle:[NSBundle bundleForClass:self.class] compatibleWithTraitCollection:nil];
            [image drawInRect:self.centerRect];
        }
            break;
        case CPDFSamplesFreehand:
        {
            CGContextSetStrokeColorWithColor(context, self.fillColor.CGColor);
            
            float tWidth = self.frame.size.width / 6.0;
            CGContextMoveToPoint(context, CGRectGetMinX(self.bounds), CGRectGetMaxY(self.bounds)/2);
            CGContextSetLineWidth(context, self.thickness);
            CGContextAddCurveToPoint(context,
                                     CGRectGetMinX(self.bounds)+tWidth,CGRectGetMaxY(self.bounds)/2+30,
                                     CGRectGetMinX(self.bounds)+tWidth*2.0,CGRectGetMaxY(self.bounds)/2-30,
                                     CGRectGetMinX(self.bounds)+tWidth*3.0,CGRectGetMaxY(self.bounds)/3);
            CGContextAddCurveToPoint(context,
                                     CGRectGetMinX(self.bounds)+tWidth*4.0,CGRectGetMaxY(self.bounds)/2+30,
                                     CGRectGetMinX(self.bounds)+tWidth*5.0,CGRectGetMaxY(self.bounds)/2-30,
                                     CGRectGetMinX(self.bounds)+tWidth*6.0,CGRectGetMaxY(self.bounds)/2);
            
            CGContextStrokePath(context);
        }
            break;
        case CPDFSamplesShapeCircle:
        {
            CGContextSetStrokeColorWithColor(context, self.fillColor.CGColor);
            CGContextSetFillColorWithColor(context, self.fillShapeColor.CGColor);
            CGFloat dashLengths[2] = {6, self.dotted};
            CGContextSetLineDash(context, 0, dashLengths, 2);
            CGContextSetLineWidth(context, self.thickness);
            CGContextAddArc(context, CGRectGetMaxX(self.bounds)/2, CGRectGetMaxY(self.bounds)/2, 30, 0, 2*PI, 0);
            CGContextDrawPath(context, kCGPathStroke);
            CGContextAddArc(context, CGRectGetMaxX(self.bounds)/2, CGRectGetMaxY(self.bounds)/2, 30, 0, 2*PI, 0);
            CGContextDrawPath(context, kCGPathFill);
        }
            break;
        case CPDFSamplesShapeRectangle:
        {
            CGContextSetStrokeColorWithColor(context, self.fillColor.CGColor);
            CGContextSetFillColorWithColor(context, self.fillShapeColor.CGColor);
            CGFloat dashLengths[2] = {6, self.dotted};
            CGContextSetLineDash(context, 0, dashLengths, 2);
            CGContextSetLineWidth(context, self.thickness);
            CGContextStrokeRect(context, self.centerRect);
            CGContextFillRect(context, self.centerRect);
        }
            break;
        case CPDFSamplesShapeArrow:
        {
            CGContextSetStrokeColorWithColor(context, self.fillColor.CGColor);
            CGContextSetFillColorWithColor(context, self.fillColor.CGColor);
            CGFloat dashLengths[2] = {6, self.dotted};
            CGContextSetLineDash(context, 0, dashLengths, 2);
            CGContextSetLineWidth(context, self.thickness);
            
            CGPoint start = CGPointMake(CGRectGetMinX(self.arrowRect), CGRectGetMaxY(self.arrowRect));
            CGPoint end = CGPointMake(CGRectGetMaxX(self.arrowRect), CGRectGetMinY(self.arrowRect));
            [self drawArrow:context startPoint:start endPoint:end];
        }
            break;
        case CPDFSamplesShapeLine:
        {
            CGContextSetStrokeColorWithColor(context, self.fillColor.CGColor);
            CGContextSetFillColorWithColor(context, self.fillColor.CGColor);
            CGFloat dashLengths[2] = {6, self.dotted};
            CGContextSetLineDash(context, 0, dashLengths, 2);
            CGContextSetLineWidth(context, self.thickness);
            
            CGPoint start = CGPointMake(CGRectGetMinX(self.arrowRect), CGRectGetMaxY(self.arrowRect));
            CGPoint end = CGPointMake(CGRectGetMaxX(self.arrowRect), CGRectGetMinY(self.arrowRect));
            [self drawArrow:context startPoint:start endPoint:end];
        }
            break;
        case CPDFSamplesFreeText:
        {
            NSString *sampleStr = @"AaBbCc";
            CGContextSetAlpha(context, self.opcity);
            UIFont *font = [UIFont fontWithName:self.fontName size:self.thickness];
            NSMutableDictionary *dic = [NSMutableDictionary dictionary];
            if (self.isBold && !(self.isUnderline)) {
                font = [UIFont boldSystemFontOfSize:self.thickness];
            } else if (self.isUnderline && !(self.isBold)) {
                [dic addEntriesFromDictionary:@{NSFontAttributeName:font, NSUnderlineStyleAttributeName:@1}];
            } else if (self.isBold && self.isUnderline) {
                font = [UIFont boldSystemFontOfSize:self.thickness];
                [dic addEntriesFromDictionary:@{NSFontAttributeName:font, NSUnderlineStyleAttributeName:@1}];
            }
            [dic addEntriesFromDictionary:@{NSFontAttributeName:font, NSForegroundColorAttributeName:self.fillColor}];
            [sampleStr drawInRect:self.arrowRect withAttributes:dic];
        }
            break;
        default:
            break;
    }
}

- (void)drawArrow:(CGContextRef)context startPoint:(CGPoint)start endPoint:(CGPoint)end {
    switch (self.arrowStyleIndex) {
        case 0:
        {
            CGContextMoveToPoint(context, start.x, start.y);
            CGContextAddLineToPoint(context, end.x, end.y);
            CGContextStrokePath(context);
        }
            break;
        case 1:
        {
            CGContextMoveToPoint(context, start.x, start.y);
            CGContextAddLineToPoint(context, end.x, end.y);
            
            CGContextMoveToPoint(context, end.x-20, end.y);
            CGContextAddLineToPoint(context, end.x, end.y);
            CGContextAddLineToPoint(context, end.x, end.y+20);
            CGContextStrokePath(context);
        }
            break;
        case 2:
        {
            CGContextMoveToPoint(context, start.x, start.y);
            CGContextAddLineToPoint(context, end.x, end.y);
            CGContextStrokePath(context);
            
            CGContextMoveToPoint(context, end.x-7.5, end.y-7.5);
            CGContextAddLineToPoint(context, end.x+15, end.y-15);
            CGContextAddLineToPoint(context, end.x+7.5, end.y+7.5);
            CGContextFillPath(context);
        }
            break;
        case 3:
        {
            CGContextMoveToPoint(context, start.x, start.y);
            CGContextAddLineToPoint(context, end.x, end.y);
            CGContextStrokePath(context);
            
            CGContextMoveToPoint(context, end.x-7.5, end.y-7.5);
            CGContextAddLineToPoint(context, end.x+7.5, end.y+7.5);
            CGContextAddLineToPoint(context, end.x+22.5, end.y-7.5);
            CGContextAddLineToPoint(context, end.x+7.5, end.y-22.5);
            CGContextFillPath(context);
        }
            break;
        case 4:
        {
            CGContextMoveToPoint(context, start.x, start.y);
            CGContextAddLineToPoint(context, end.x, end.y);
            CGContextStrokePath(context);
            
            CGContextAddArc(context, end.x+7.5, end.y-7.5, 10, 0, 2*3.14159265358979323846, 0);
            CGContextDrawPath(context, kCGPathStroke);
        }
            break;
        case 5:
        {
            CGContextMoveToPoint(context, start.x, start.y);
            CGContextAddLineToPoint(context, end.x, end.y);
            CGContextStrokePath(context);
            
            CGContextMoveToPoint(context, end.x, end.y);
            CGContextAddLineToPoint(context, end.x+15, end.y);
            CGContextAddLineToPoint(context, end.x+15, end.y-15);
            CGContextAddLineToPoint(context, end.x, end.y-15);
            CGContextFillPath(context);
        }
            break;
        default:
            break;
    }
}

@end
